# Class: device_hiera::netdev_interfaces
# =======================================
#
# Utilize Hiera data to simplify NetDev netdev_interface configuration.
#
# Parameters
# ----------
#
# None yet.
#
# Variables
# ----------
#
# Here you should define a list of variables that this module would require.
#
# * `$defaults`
#  This contains a hash of default netdev_interface configuration values 
#  gathered from Hiera data merge.
#
# * `$ports`
#  This contains a hash with String keys named for slot names,
#  and String values containing a range of port numbers.
#
# * `$custom`
#  This contains a hash with String keys for netdev_interface names 
#  that receive custom configuration. Values are another hash
#  of configuration settings.
#
# Examples
#
# @example Hiera configuration for device_hiera::netdev_interfaces
#
#  classes:
#  - device_hiera
#  
#  device_hiera::resources:
#  - netdev_interfaces
#
#  device_hiera::defaults::netdev_interface:
#    family: 'inet'
#    unit  : '0'
#    duplex: 'full'
#    enable: 'true'
#    mtu   : '1500'
# 
#  device_hiera::netdev_interfaces::ports:
#    - 'ge-1/0/': '1-8'
#    - 'ge-2/0/': '1-48'
# 
#  device_hiera::netdev_interfaces::custom:
#    'ge-4/0/1':
#      description: 'Uplink to core'
#      address    : '192.168.51.254'
#      speed      : '1g'
#
class device_hiera::netdev_interfaces(
) {

  # Determine the default netdev_interface profile
  $defaults = hiera_hash( 'device_hiera::defaults::netdev_interface', {} )

  # Now get a list of ports
  $ports = hiera_array( 'device_hiera::netdev_interfaces::ports', {} )
  $custom = hiera_hash( 'device_hiera::netdev_interfaces::custom', {} )

  # Process every listed port unless it has a custom config
  $ports.each() |$slot| {
    $slot.each() |$prefix,$ports| {
      $range = $ports.scanf('%i-%i') |$values| {
        $min = $values[0]
        $max = $values[1]
        unless $min > 0 {
          fail "Invalid port range #{ports}"
          [-1,-1]
        }
        unless $max >= $min {
          fail "Invalid port range #{ports}"
          [-1,-1]
        }
        [$min,$max]
      }

      Integer[ $range[0], $range[1] ].each |$portnum| {
        # assumes that prefix contains any necessary separator
        $intname = "${prefix}${portnum}"

        # If the port doesn't have a custom config, give it the default config
        if $custom[$intname] == undef {
          $reshash = { $intname => $defaults, }
          create_resources( netdev_interface, $reshash )
        }
      }
    }
  }

  # Process all customized port configs
  $custom.each() |$interface,$intconfig| {
    # don't duplicate description to custom port configs
    $clean_defaults = $defaults - ['description']

    # Now create the resource
    $reshash = { $interface => $intconfig, }
    create_resources( netdev_interface, $reshash, $clean_defaults )
  }
}
